@@ -1,5 +1,5 @@
 #![forbid(unsafe_code)]
-#![warn(future_incompatible, rust_2018_idioms, single_use_lifetimes, unreachable_pub)]
+#![warn(rust_2018_idioms, single_use_lifetimes, unreachable_pub)]
 #![warn(clippy::default_trait_access, clippy::wildcard_imports)]
 
 use std::{
@@ -8,37 +8,46 @@ use std::{
 };
 
 use anyhow::{bail, Context as _, Result};
+use clap::{AppSettings, ArgSettings, Clap};
 use parse_changelog::Parser;
-use structopt::{clap::AppSettings, StructOpt};
 
-/// Parses changelog and returns a release note for the specified version.
-#[derive(StructOpt)]
-#[structopt(
-    rename_all = "kebab-case",
-    setting = AppSettings::DeriveDisplayOrder,
-    setting = AppSettings::UnifiedHelpMessage,
+const ABOUT: &str = "Simple changelog parser, written in Rust.
+
+Parses changelog and returns a release note for the specified version.
+
+Use -h for short descriptions and --help for more details.";
+
+const MAX_TERM_WIDTH: usize = 100;
+
+#[derive(Clap)]
+#[clap(
+    about(ABOUT),
+    version,
+    max_term_width(MAX_TERM_WIDTH),
+    setting(AppSettings::DeriveDisplayOrder),
+    setting(AppSettings::UnifiedHelpMessage)
 )]
 struct Args {
     /// Path to the changelog file (use '-' for standard input).
-    #[structopt(value_name = "PATH")]
+    #[clap(value_name = "PATH", setting(ArgSettings::ForbidEmptyValues))]
     path: String,
     /// Specify version (by default, select the latest release).
-    #[structopt(value_name = "VERSION")]
-    version: Option<String>,
+    #[clap(value_name = "VERSION", setting(ArgSettings::ForbidEmptyValues))]
+    release: Option<String>,
     /// Returns title instead of notes.
-    #[structopt(short, long)]
+    #[clap(short, long)]
     title: bool,
     /// Returns JSON representation of all releases in changelog.
-    #[structopt(long, conflicts_with = "version", conflicts_with = "title")]
+    #[clap(long, conflicts_with = "version", conflicts_with = "title")]
     json: bool,
     /// Specify version format.
-    #[structopt(long, value_name = "PATTERN")]
+    #[clap(long, value_name = "PATTERN", setting(ArgSettings::ForbidEmptyValues))]
     version_format: Option<String>,
-    /// Alias for --prefix-format.
-    #[structopt(long, value_name = "PATTERN")]
-    prefix: Option<String>,
     /// Specify prefix format.
-    #[structopt(long, value_name = "PATTERN", conflicts_with = "prefix")]
+    ///
+    /// By default only "v", "Version ", "Release ", and "" (no prefix) are
+    /// allowed as prefixes.
+    #[clap(long, value_name = "PATTERN", visible_alias = "prefix")]
     prefix_format: Option<String>,
 }
 
@@ -50,20 +59,19 @@ fn main() {
 }
 
 fn try_main() -> Result<()> {
-    let args = Args::from_args();
+    let args = Args::parse();
 
     let mut parser = Parser::new();
     if let Some(version_format) = &args.version_format {
         parser.version_format(version_format)?;
     }
-    if let Some(prefix_format) = args.prefix_format.as_ref().map_or(args.prefix.as_ref(), Some) {
+    if let Some(prefix_format) = &args.prefix_format {
         parser.prefix_format(prefix_format)?;
     }
 
     let text = if args.path == "-" {
         let mut buf = String::new();
-        let stdin = io::stdin();
-        stdin.lock().read_to_string(&mut buf).context("failed to read standard input")?;
+        io::stdin().read_to_string(&mut buf).context("failed to read standard input")?;
         buf
     } else {
         fs::read_to_string(&args.path).with_context(|| format!("failed to read {}", args.path))?
@@ -78,7 +86,7 @@ fn try_main() -> Result<()> {
         return Ok(());
     }
 
-    let release = if let Some(version) = args.version.as_deref() {
+    let release = if let Some(version) = args.release.as_deref() {
         if let Some(release) = changelog.get(version) {
             release
         } else {
@@ -100,15 +108,18 @@ mod tests {
     use std::{env, fs, path::Path, process::Command};
 
     use anyhow::Result;
-    use structopt::StructOpt;
+    use clap::IntoApp;
     use tempfile::Builder;
 
-    use super::Args;
+    use super::{Args, MAX_TERM_WIDTH};
 
-    fn get_long_help() -> Result<String> {
-        let mut app = Args::clap();
+    fn get_help(long: bool) -> Result<String> {
         let mut buf = vec![];
-        app.write_long_help(&mut buf)?;
+        if long {
+            Args::into_app().term_width(MAX_TERM_WIDTH).write_long_help(&mut buf)?;
+        } else {
+            Args::into_app().term_width(MAX_TERM_WIDTH).write_help(&mut buf)?;
+        }
         let mut out = String::new();
         for mut line in String::from_utf8(buf)?.lines() {
             if let Some(new) = line.trim_end().strip_suffix(env!("CARGO_PKG_VERSION")) {
@@ -149,13 +160,19 @@ mod tests {
 
     #[test]
     fn long_help() {
-        let actual = get_long_help().unwrap();
+        let actual = get_help(true).unwrap();
         assert_diff("tests/long-help.txt", actual);
     }
 
+    #[test]
+    fn short_help() {
+        let actual = get_help(false).unwrap();
+        assert_diff("tests/short-help.txt", actual);
+    }
+
     #[test]
     fn update_readme() -> Result<()> {
-        let new = get_long_help()?;
+        let new = get_help(true)?;
         let path = &Path::new(env!("CARGO_MANIFEST_DIR")).join("README.md");
         let base = fs::read_to_string(path)?;
         let mut out = String::with_capacity(base.capacity());
@@ -170,7 +187,6 @@ mod tests {
                 out.push_str("```console\n");
                 out.push_str("$ parse-changelog --help\n");
                 out.push_str(&new);
-                out.push('\n');
                 for line in &mut lines {
                     if line == "<!-- readme-long-help:end -->" {
                         out.push_str("```\n");
